@@ -38,7 +38,6 @@ __docformat__ = 'epytext en'
 import re
 import ast
 import functools
-import sys
 from inspect import signature
 from typing import Any, AnyStr, Union, Callable, Dict, Iterable, Sequence, Optional, List, Tuple, cast
 
@@ -199,7 +198,9 @@ class ColorizedPyvalRepr(ParsedRstDocstring):
     def to_stan(self, docstring_linker: DocstringLinker) -> Tag:
         return Tag('code')(super().to_stan(docstring_linker))
 
-def colorize_pyval(pyval: Any, linelen:Optional[int], maxlines:int, linebreakok:bool=True, refmap:Optional[Dict[str, str]]=None) -> ColorizedPyvalRepr:
+def colorize_pyval(pyval: Any, linelen:Optional[int], maxlines:int, 
+                   linebreakok:bool=True, refmap:Optional[Dict[str, str]]=None, 
+                   is_annotation: bool = False) -> ColorizedPyvalRepr:
     """
     Get a L{ColorizedPyvalRepr} instance for this piece of ast. 
 
@@ -209,14 +210,15 @@ def colorize_pyval(pyval: Any, linelen:Optional[int], maxlines:int, linebreakok:
         This can be used for cases the where the linker might be wrong, obviously this is just a workaround.
     @return: A L{ColorizedPyvalRepr} describing the given pyval.
     """
-    return PyvalColorizer(linelen=linelen, maxlines=maxlines, linebreakok=linebreakok, refmap=refmap).colorize(pyval)
+    return PyvalColorizer(linelen=linelen, maxlines=maxlines, linebreakok=linebreakok, 
+                          refmap=refmap, is_annotation=is_annotation).colorize(pyval)
 
-def colorize_inline_pyval(pyval: Any, refmap:Optional[Dict[str, str]]=None) -> ColorizedPyvalRepr:
+def colorize_inline_pyval(pyval: Any, refmap:Optional[Dict[str, str]]=None, is_annotation: bool = False) -> ColorizedPyvalRepr:
     """
     Used to colorize type annotations and parameters default values.
     @returns: C{L{colorize_pyval}(pyval, linelen=None, linebreakok=False)}
     """
-    return colorize_pyval(pyval, linelen=None, maxlines=1, linebreakok=False, refmap=refmap)
+    return colorize_pyval(pyval, linelen=None, maxlines=1, linebreakok=False, refmap=refmap, is_annotation=is_annotation)
 
 def _get_str_func(pyval:  AnyStr) -> Callable[[str], AnyStr]:
     func = cast(Callable[[str], AnyStr], str if isinstance(pyval, str) else \
@@ -261,14 +263,17 @@ def _bytes_escape(b: bytes) -> str:
 
 class PyvalColorizer:
     """
-    Syntax highlighter for Python values.
+    Syntax highlighter for Python AST (and some builtins types).
     """
 
-    def __init__(self, linelen:Optional[int], maxlines:int, linebreakok:bool=True, refmap:Optional[Dict[str, str]]=None):
+    def __init__(self, linelen:Optional[int], maxlines:int, linebreakok:bool=True, 
+                 refmap:Optional[Dict[str, str]]=None, is_annotation: bool = False):
         self.linelen: Optional[int] = linelen if linelen!=0 else None
         self.maxlines: Union[int, float] = maxlines if maxlines!=0 else float('inf')
         self.linebreakok = linebreakok
         self.refmap = refmap if refmap is not None else {}
+        self.is_annotation = is_annotation
+
         # some edge cases require to compute the precedence ahead of time and can't be 
         # easily done with access only to the parent node of some operators.
         self.explicit_precedence:Dict[ast.AST, int] = {}
@@ -284,7 +289,7 @@ class PyvalColorizer:
     NUMBER_TAG = None                # ints, floats, etc
     QUOTE_TAG = 'variable-quote'     # Quotes around strings.
     STRING_TAG = 'variable-string'   # Body of string literals
-    LINK_TAG = 'variable-link'       # Links to other documentables, extracted from AST names and attributes.
+    LINK_TAG = None       # Links, we don't use an explicit class here, but in node2stan.
     ELLIPSIS_TAG = 'variable-ellipsis'
     LINEWRAP_TAG = 'variable-linewrap'
     UNKNOWN_TAG = 'variable-unknown'
@@ -517,35 +522,9 @@ class PyvalColorizer:
     #   comparators, 
     #   generator expressions, 
     #   Slice and ExtSlice
-
-    @staticmethod
-    def _is_ast_constant(node: ast.AST) -> bool:
-        if sys.version_info[:2] >= (3, 8):
-            return isinstance(node, ast.Constant)
-        else:
-            # TODO: remove me when python3.7 is not supported anymore
-            return isinstance(node, (ast.Num, ast.Str, ast.Bytes, 
-                    ast.Constant, ast.NameConstant, ast.Ellipsis))
-    @staticmethod
-    def _get_ast_constant_val(node: ast.AST) -> Any:
-        # Deprecated since version 3.8: Replaced by Constant
-        if sys.version_info[:2] >= (3, 8):
-            if isinstance(node, ast.Constant):
-                return node.value
-        else:
-            # TODO: remove me when python3.7 is not supported anymore
-            if isinstance(node, ast.Num): 
-                return(node.n)
-            if isinstance(node, (ast.Str, ast.Bytes)):
-                return(node.s)
-            if isinstance(node, (ast.Constant, ast.NameConstant)):
-                return(node.value)
-            if isinstance(node, ast.Ellipsis):
-                return(...)
-        raise RuntimeError(f'expected a constant: {ast.dump(node)}')
         
-    def _colorize_ast_constant(self, pyval: ast.AST, state: _ColorizerState) -> None:
-        val = self._get_ast_constant_val(pyval)
+    def _colorize_ast_constant(self, pyval: ast.Constant, state: _ColorizerState) -> None:
+        val = pyval.value
         # Handle elipsis
         if val != ...:
             self._colorize(val, state)
@@ -560,7 +539,7 @@ class PyvalColorizer:
         except StopIteration:
             Parentage().visit(pyval)
 
-        if self._is_ast_constant(pyval): 
+        if isinstance(pyval, ast.Constant): 
             self._colorize_ast_constant(pyval, state)
         elif isinstance(pyval, ast.UnaryOp):
             self._colorize_ast_unary_op(pyval, state)
@@ -666,9 +645,6 @@ class PyvalColorizer:
         self._colorize(node.value, state)
 
         sub: ast.AST = node.slice
-        if sys.version_info < (3,9) and isinstance(sub, ast.Index):
-            # In Python < 3.9, non-slices are always wrapped in an Index node.
-            sub = sub.value
         self._output('[', self.GROUP_TAG, state)
         self._set_precedence(op_util.Precedence.Subscript, node)
         self._set_precedence(op_util.Precedence.Index, sub)
@@ -712,11 +688,11 @@ class PyvalColorizer:
         ast_pattern = args.arguments['pattern']
 
         # Cannot colorize regex
-        if not self._is_ast_constant(ast_pattern):
+        if not isinstance(ast_pattern, ast.Constant):
             self._colorize_ast_call_generic(node, state)
             return
 
-        pat = self._get_ast_constant_val(ast_pattern)
+        pat = ast_pattern.value
         
         # Just in case regex pattern is not valid type
         if not isinstance(pat, (bytes, str)):
@@ -755,8 +731,7 @@ class PyvalColorizer:
             # if there are required since we don;t have support for all operators 
             # See TODO comment in _OperatorDelimiter.
             source = unparse(pyval).strip()
-            if sys.version_info > (3,9) and isinstance(pyval, 
-                    (ast.IfExp, ast.Compare, ast.Lambda)) and len(state.stack)>1:
+            if isinstance(pyval, (ast.IfExp, ast.Compare, ast.Lambda)) and len(state.stack)>1:
                 source = f'({source})'
         except Exception: #  No defined handler for node of type <type>
             state.result.append(self.UNKNOWN_REPR)
@@ -1017,16 +992,21 @@ class PyvalColorizer:
             if (self.linelen is None or 
                 state.charpos + segment_len <= self.linelen 
                 or link is True 
-                or css_class in ('variable-quote',)):
+                or css_class in (self.QUOTE_TAG,)):
 
                 state.charpos += segment_len
 
                 if link is True:
                     # Here, we bypass the linker if refmap contains the segment we're linking to. 
-                    # The linker can be problematic because it has some design blind spots when the same name is declared in the imports and in the module body.
+                    # The linker can be problematic because it has some design blind spots when 
+                    # the same name is declared in the imports and in the module body.
                     
                     # Note that the argument name is 'refuri', not 'refuid. 
-                    element = obj_reference('', segment, refuri=self.refmap.get(segment, segment))
+                    element = obj_reference('', segment, 
+                                            refuri=self.refmap.get(segment, segment))
+                    if self.is_annotation:
+                        # Don't set the attribute if it's not True.
+                        element.attributes['is_annotation'] = True
                 elif css_class is not None:
                     element = nodes.inline('', segment, classes=[css_class])
                 else:
