@@ -5,6 +5,7 @@
 """
 Support for L{twisted.python.deprecate}.
 """
+from __future__ import annotations
 
 import ast
 import inspect
@@ -54,25 +55,25 @@ class ModuleVisitor(extensions.ModuleVisitorExt):
         """
         Called after a class definition is visited.
         """
+        current = self.visitor.builder.current
         try:
-            cls = self.visitor.builder.current.contents[node.name]
+            cls = current.contents[node.name]
         except KeyError:
             # Classes inside functions are ignored.
             return
-        assert isinstance(cls, model.Class)
         getDeprecated(cls, node.decorator_list)
 
     def depart_FunctionDef(self, node:ast.FunctionDef) -> None:
         """
         Called after a function definition is visited.
         """
+        current = self.visitor.builder.current
         try:
             # Property or Function
-            func = self.visitor.builder.current.contents[node.name]
+            func = current.contents[node.name]
         except KeyError:
             # Inner functions are ignored.
             return
-        assert isinstance(func, (model.Function, model.Attribute))
         getDeprecated(func, node.decorator_list)
 
 _incremental_Version_signature = inspect.signature(Version)
