@@ -8,16 +8,18 @@ Forked from ``sphinx.ext.napoleon.docstring``.
     :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+from __future__ import annotations
+
 import collections
 from enum import Enum, auto
 import re
 
 from functools import partial
-from typing import Any, Callable, Deque, Dict, Iterator, List, Optional, Tuple, Union
+from typing import Any, Callable, Deque, Dict, Iterator, List, Literal, Optional, Tuple, Union
 
 import attr
 
-from pydoctor.napoleon.iterators import modify_iter, peek_iter
+from .iterators import modify_iter, peek_iter
 
 __docformat__ = "numpy en"
 
@@ -27,8 +29,10 @@ _google_typed_arg_regex = re.compile(r"(.+?)\(\s*(.*[^\s]+)\s*\)")
 _numpy_section_regex = re.compile(r'^[=\-`:\'"~^_*+#<>]{2,}\s*$')
 _single_colon_regex = re.compile(r"(?<!:):(?!:)")
 _xref_or_code_regex = re.compile(
-    r"((?::(?:[a-zA-Z0-9]+[\-_+:.])*[a-zA-Z0-9]+:`.+?`)|" r"(?:``.+?``))"
-)
+    r'((?::(?:[a-zA-Z0-9]+[\-_+:.])*[a-zA-Z0-9]+:`.+?`)|'
+    r'(?:``.+?``)|'
+    # r'(?::meta .+:.*)|' # 'meta' is not a supported field by pydoctor at the moment.
+    r'(?:`.+?\s*(?<!\x00)<.*?>`))')
 _xref_regex = re.compile(r"(?:(?::(?:[a-zA-Z0-9]+[\-_+:.])*[a-zA-Z0-9]+:)?`.+?`)")
 _bullet_list_regex = re.compile(r"^(\*|\+|\-)(\s+\S|\s*$)")
 _enumerated_list_regex = re.compile(
@@ -254,8 +258,8 @@ class TypeDocstring:
         Example
         -------
         >>> tokens = ["{", "1", ", ", "2", "}"]
-        >>> ann._recombine_set_tokens(tokens)
-        ... ["{1, 2}"]
+        >>> TypeDocstring._recombine_set_tokens(tokens)
+        ['{1, 2}']
         """
         token_queue = collections.deque(tokens)
         keywords = ("optional", "default")
@@ -475,42 +479,40 @@ class GoogleDocstring:
     Example
     -------
 
-    .. python::
-        >>> from pydoctor.napoleon import GoogleDocstring
-        >>> docstring = '''One line summary.
-        ...
-        ... Extended description.
-        ...
-        ... Args:
-        ...   arg1(int): Description of `arg1`
-        ...   arg2(str): Description of `arg2`
-        ... Returns:
-        ...   str: Description of return value.
-        ... '''
-        >>> print(GoogleDocstring(docstring))
-        One line summary.
-
-        Extended description.
-
-        :param arg1: Description of `arg1`
-        :type arg1: int
-        :param arg2: Description of `arg2`
-        :type arg2: str
-
-        :returns: Description of return value.
-        :returntype: str
-        >>> print(GoogleDocstring(docstring, process_type_fields=True))
-        One line summary.
-
-        Extended description.
-
-        :param arg1: Description of `arg1`
-        :type arg1: `int`
-        :param arg2: Description of `arg2`
-        :type arg2: `str`
-
-        :returns: Description of return value.
-        :returntype: `str`
+    >>> docstring = '''One line summary.
+    ...
+    ... Extended description.
+    ...
+    ... Args:
+    ...   arg1(int): Description of `arg1`
+    ...   arg2(str): Description of `arg2`
+    ... Returns:
+    ...   str: Description of return value.
+    ... '''
+    >>> print(GoogleDocstring(docstring))
+    One line summary.
+    <BLANKLINE>
+    Extended description.
+    <BLANKLINE>
+    :param arg1: Description of `arg1`
+    :type arg1: int
+    :param arg2: Description of `arg2`
+    :type arg2: str
+    <BLANKLINE>
+    :returns: Description of return value.
+    :returntype: str
+    >>> print(GoogleDocstring(docstring, process_type_fields=True))
+    One line summary.
+    <BLANKLINE>
+    Extended description.
+    <BLANKLINE>
+    :param arg1: Description of `arg1`
+    :type arg1: `int`
+    :param arg2: Description of `arg2`
+    :type arg2: `str`
+    <BLANKLINE>
+    :returns: Description of return value.
+    :returntype: `str`
 
     """
 
@@ -522,7 +524,7 @@ class GoogleDocstring:
 
     # overriden
     def __init__(self, docstring: Union[str, List[str]], 
-        is_attribute: bool = False, 
+        what: Literal['function', 'module', 'class', 'attribute'] | None = None,
         process_type_fields: bool = False,
     ) -> None:
         """
@@ -531,15 +533,13 @@ class GoogleDocstring:
         docstring : str or list of str
             The docstring to parse, given either as a string or split into
             individual lines.
-        is_attribute: bool
-            If the documented object is an attribute,
-            it will use the `_parse_attribute_docstring` method.
+        what: 
+            Optional string representing the type of object we're documenting.
         process_type_fields: bool
             Whether to process the type fields or to leave them untouched (default) in order to be processed later.
             Value ``process_type_fields=False`` is currently only used in the tests.
         """
-
-        self._is_attribute = is_attribute
+        self._what = what
         self._process_type_fields = process_type_fields
         
         if isinstance(docstring, str):
@@ -953,8 +953,7 @@ class GoogleDocstring:
                 indent = self._get_indent(line)
                 if min_indent is None:
                     min_indent = indent
-                # mypy get error: Statement is unreachable  [unreachable]
-                elif indent < min_indent:  # type: ignore
+                elif indent < min_indent:
                     min_indent = indent
         return min_indent or 0
 
@@ -1007,12 +1006,12 @@ class GoogleDocstring:
             )
         )
 
-    # overriden: call _parse_attribute_docstring if self._is_attribute is True
+    # overriden: call _parse_attribute_docstring if the object is an attribute
     # and add empty blank lines when required
     def _parse(self) -> None:
         self._parsed_lines = self._consume_empty()
 
-        if self._is_attribute:
+        if self._what == 'attribute':
             # Implicit stop using StopIteration no longer allowed in
             # Python 3.7; see PEP 479
             res = []  # type: List[str]
@@ -1065,9 +1064,10 @@ class GoogleDocstring:
     # TODO: add 'vartype' and 'kwtype' as aliases of 'type' and use them here to output
     #       the most correct reStructuredText.
     def _parse_attributes_section(self, section: str) -> List[str]:
+        fieldtag = 'var' if self._what == 'module' else 'ivar'
         lines = []
         for f in self._consume_fields():
-            field = f":ivar {f.name}: "
+            field = f":{fieldtag} {f.name}: "
             lines.extend(self._format_block(field, f.content))
             if f.type:
                 lines.append(f":type {f.name}: {self._convert_type(f.type, lineno=f.lineno)}")
@@ -1341,48 +1341,46 @@ class NumpyDocstring(GoogleDocstring):
 
     Example
     -------
-    
-    .. python::
-        >>> from pydoctor.napoleon import NumpyDocstring
-        >>> docstring = '''One line summary.
-        ...
-        ... Extended description.
-        ...
-        ... Parameters
-        ... ----------
-        ... arg1 : int
-        ...     Description of `arg1`
-        ... arg2 : str
-        ...     Description of `arg2`
-        ... Returns
-        ... -------
-        ... str
-        ...     Description of return value.
-        ... '''
-        >>> print(NumpyDocstring(docstring))
-        One line summary.
 
-        Extended description.
-
-        :param arg1: Description of `arg1`
-        :type arg1: int
-        :param arg2: Description of `arg2`
-        :type arg2: str
-
-        :returns: Description of return value.
-        :returntype: str
-        >>> print(NumpyDocstring(docstring, process_type_fields=True))
-        One line summary.
-
-        Extended description.
-
-        :param arg1: Description of `arg1`
-        :type arg1: `int`
-        :param arg2: Description of `arg2`
-        :type arg2: `str`
-
-        :returns: Description of return value.
-        :returntype: `str`
+    >>> docstring = '''One line summary.
+    ...
+    ... Extended description.
+    ...
+    ... Parameters
+    ... ----------
+    ... arg1 : int
+    ...     Description of `arg1`
+    ... arg2 : str
+    ...     Description of `arg2`
+    ... Returns
+    ... -------
+    ... str
+    ...     Description of return value.
+    ... '''
+    >>> print(NumpyDocstring(docstring))
+    One line summary.
+    <BLANKLINE>
+    Extended description.
+    <BLANKLINE>
+    :param arg1: Description of `arg1`
+    :type arg1: int
+    :param arg2: Description of `arg2`
+    :type arg2: str
+    <BLANKLINE>
+    :returns: Description of return value.
+    :returntype: str
+    >>> print(NumpyDocstring(docstring, process_type_fields=True))
+    One line summary.
+    <BLANKLINE>
+    Extended description.
+    <BLANKLINE>
+    :param arg1: Description of `arg1`
+    :type arg1: `int`
+    :param arg2: Description of `arg2`
+    :type arg2: `str`
+    <BLANKLINE>
+    :returns: Description of return value.
+    :returntype: `str`
 
     """
 
