@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from typing import TYPE_CHECKING, List
 
 from twisted.web.iweb import ITemplateLoader
@@ -49,7 +51,7 @@ class AttributeChild(TemplateElement):
     
     @renderer
     def decorator(self, request: object, tag: Tag) -> "Flattenable":
-        return list(format_decorators(self.ob))
+        return format_decorators(self.ob)
 
     @renderer
     def attribute(self, request: object, tag: Tag) -> "Flattenable":
@@ -61,8 +63,8 @@ class AttributeChild(TemplateElement):
 
     @renderer
     def sourceLink(self, request: object, tag: Tag) -> "Flattenable":
-        if self.ob.sourceHref:
-            return tag.fillSlots(sourceHref=self.ob.sourceHref)
+        if self.ob.source_href:
+            return tag.fillSlots(sourceHref=self.ob.source_href)
         else:
             return ()
 
