@@ -29,7 +29,13 @@ def srclink(o: model.Documentable) -> Optional[str]:
     """
     Get object source code URL, i.e. hosted on github. 
     """
-    return o.sourceHref
+    return o.source_href
+
+def package_srclinks(o: model.Package) -> list[str]:
+    """
+    Get (namespace) package source code URLs, i.e. hosted on github. 
+    """
+    return o.source_hrefs
 
 def css_class(o: model.Documentable) -> str:
     """
@@ -107,7 +113,8 @@ def source_order_func(o: model.Documentable) -> Tuple[Any, ...]:
         # last implicit orderring is the order of insertion.
 
 def _map_kind(kind: model.DocumentableKind) -> model.DocumentableKind:
-    if kind == model.DocumentableKind.PACKAGE:
+    if kind in (model.DocumentableKind.PACKAGE, 
+                model.DocumentableKind.NAMESPACE_PACKAGE):
         # packages and modules should be listed together
         return model.DocumentableKind.MODULE
     return kind
