@@ -7,9 +7,17 @@
 
 from typing import List
 from pydoctor.epydoc.markup import ParseError, ParsedDocstring, get_parser_by_name
+import pydoctor.epydoc.markup
 
 def parse_docstring(doc: str, markup: str, processtypes: bool = False) -> ParsedDocstring:
+    
+    parse = get_parser_by_name(markup)
+    if processtypes:
+        if markup in ('google','numpy'):
+            raise AssertionError("don't process types twice.")
+        parse = pydoctor.epydoc.markup.processtypes(parse)
+    
     errors: List[ParseError] = []
-    parsed = get_parser_by_name(markup)(doc, errors, processtypes)
+    parsed = parse(doc, errors)
     assert not errors, [f"{e.linenum()}:{e.descr()}" for e in errors]
     return parsed
