@@ -10,12 +10,11 @@ import pytest
 from pydoctor.epydoc.markup._pyval_repr import PyvalColorizer, colorize_inline_pyval
 from pydoctor.test import NotFoundLinker
 from pydoctor.stanutils import flatten, flatten_text, html2stan
-from pydoctor.node2stan import gettext
 
-def color(v: Any, linebreakok:bool=True, maxlines:int=5, linelen:int=40) -> str:
-    colorizer = PyvalColorizer(linelen=linelen, linebreakok=linebreakok, maxlines=maxlines)
+def color(v: Any, linebreakok:bool=True, maxlines:int=5, linelen:int=40, is_annotation: bool = False) -> str:
+    colorizer = PyvalColorizer(linelen=linelen, linebreakok=linebreakok, maxlines=maxlines, is_annotation=is_annotation)
     parsed_doc = colorizer.colorize(v)
-    return parsed_doc.to_node().pformat() #type: ignore
+    return parsed_doc.to_node().pformat()
 
 def colorhtml(v: Any, linebreakok:bool=True, maxlines:int=5, linelen:int=40) -> str:
     colorizer = PyvalColorizer(linelen=linelen, linebreakok=linebreakok, maxlines=maxlines)
@@ -566,7 +565,7 @@ def test_ast_bin_op() -> None:
     2.3*6
     """)))) == """<document source="pyval_repr">
     2.3
-    *
+     * 
     6\n"""
 
     assert color(extract_expr(ast.parse(dedent("""
@@ -574,56 +573,56 @@ def test_ast_bin_op() -> None:
     """)))) == """<document source="pyval_repr">
     (
     3
-    -
+     - 
     6
     )
-    *
+     * 
     2\n"""
 
     assert color(extract_expr(ast.parse(dedent("""
     101//4+101%4
     """)))) == """<document source="pyval_repr">
     101
-    //
+     // 
     4
-    +
+     + 
     101
-    %
+     % 
     4\n"""
 
     assert color(extract_expr(ast.parse(dedent("""
     1 & 0
     """)))) == """<document source="pyval_repr">
     1
-    &
+     & 
     0\n"""
 
     assert color(extract_expr(ast.parse(dedent("""
     1 | 0
     """)))) == """<document source="pyval_repr">
     1
-    |
+     | 
     0\n"""
 
     assert color(extract_expr(ast.parse(dedent("""
     1 ^ 0
     """)))) == """<document source="pyval_repr">
     1
-    ^
+     ^ 
     0\n"""
 
     assert color(extract_expr(ast.parse(dedent("""
     1 << 0
     """)))) == """<document source="pyval_repr">
     1
-    <<
+     << 
     0\n"""
     
     assert color(extract_expr(ast.parse(dedent("""
     1 >> 0
     """)))) == """<document source="pyval_repr">
     1
-    >>
+     >> 
     0\n"""
 
     assert color(extract_expr(ast.parse(dedent("""
@@ -631,7 +630,7 @@ def test_ast_bin_op() -> None:
     """)))) == """<document source="pyval_repr">
     <obj_reference refuri="H">
         H
-    @
+     @ 
     <obj_reference refuri="beta">
         beta\n"""
 
@@ -642,20 +641,20 @@ def test_operator_precedences() -> None:
     """)))) == """<document source="pyval_repr">
     (
     2
-    **
+     ** 
     3
     )
-    **
+     ** 
     2\n"""
 
     assert color(extract_expr(ast.parse(dedent("""
     2 ** 3 ** 2
     """)))) == """<document source="pyval_repr">
     2
-    **
+     ** 
     (
     3
-    **
+     ** 
     2
     )\n"""
 
@@ -664,12 +663,12 @@ def test_operator_precedences() -> None:
     """)))) == """<document source="pyval_repr">
     (
     1
-    +
+     + 
     2
     )
-    *
+     * 
     3
-    /
+     / 
     4\n"""
 
     assert color(extract_expr(ast.parse(dedent("""
@@ -677,12 +676,12 @@ def test_operator_precedences() -> None:
     """)))) == """<document source="pyval_repr">
     (
     1
-    +
+     + 
     2
     )
-    *
+     * 
     3
-    /
+     / 
     4\n"""
 
     assert color(extract_expr(ast.parse(dedent("""
@@ -690,25 +689,25 @@ def test_operator_precedences() -> None:
     """)))) == """<document source="pyval_repr">
     (
     1
-    +
+     + 
     2
     )
-    *
+     * 
     3
-    /
+     / 
     4\n"""
 
     assert color(extract_expr(ast.parse(dedent("""
     1 + 2 * 3 / 4 - 1
     """)))) == """<document source="pyval_repr">
     1
-    +
+     + 
     2
-    *
+     * 
     3
-    /
+     / 
     4
-    -
+     - 
     1\n"""
 
 def test_ast_bool_op() -> None:
@@ -783,7 +782,7 @@ def test_ast_list_tuple() -> None:
     <wbr>
     11
     ]
-    +
+     + 
     [
     <wbr>
     99
@@ -847,7 +846,7 @@ def test_ast_dict() -> None:
     """
     assert color(extract_expr(ast.parse(dedent("""
     {'1':33, '2':[1,2,3,{7:'oo'*20}]}
-    """)))) == """<document source="pyval_repr">
+    """))), linelen=45) == """<document source="pyval_repr">
     {
     <wbr>
     <inline classes="variable-quote">
@@ -888,7 +887,7 @@ def test_ast_dict() -> None:
         oo
     <inline classes="variable-quote">
         '
-    *
+     * 
     20
     }
     ]
@@ -1020,14 +1019,8 @@ def test_ast_slice() -> None:
         o
     [
     <wbr>
-    x:y, (z)
-    ]\n""" if sys.version_info < (3,9) else """<document source="pyval_repr">
-    <obj_reference refuri="o">
-        o
-    [
-    <wbr>
     x:y
-    ,
+    , 
     <wbr>
     <obj_reference refuri="z">
         z
@@ -1160,7 +1153,7 @@ def color_re(s: Union[bytes, str],
     val = colorizer.colorize(extract_expr(ast.parse(f"re.compile({repr(s)})")))
 
     if check_roundtrip:
-        raw_text = ''.join(gettext(val.to_node()))
+        raw_text = val.to_text()
         re_begin = 13
         raw_string = True
 
@@ -1428,7 +1421,7 @@ def color2(v: Any, linelen:int=50) -> str:
     """
     colorizer = PyvalColorizer(linelen=linelen, maxlines=5)
     colorized = colorizer.colorize(v)
-    text1 = ''.join(gettext(colorized.to_node()))
+    text1 = colorized.to_text()
     text2 = flatten_text(html2stan(flatten(colorized.to_stan(NotFoundLinker()))))
     assert text1 == text2
     return text2
@@ -1473,7 +1466,7 @@ def test_summary() -> None:
     """
     summarizer = PyvalColorizer(linelen=60, maxlines=1, linebreakok=False)
     def summarize(v:Any) -> str:
-        return(''.join(gettext(summarizer.colorize(v).to_node())))
+        return summarizer.colorize(v).to_text()
 
     assert summarize(list(range(100))) == "[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16..."
     assert summarize('hello\nworld') == r"'hello\nworld'"
@@ -1507,13 +1500,15 @@ def check_src_roundtrip(src:str, subtests:Any) -> None:
 
 def test_expressions_parens(subtests:Any) -> None:
     check_src = partial(check_src_roundtrip, subtests=subtests)
-    check_src("1<<(10|1)<<1")
-    check_src("int|float|complex|None")
-    check_src("1+1")
-    check_src("1+2/3")
-    check_src("(1+2)/3")
-    check_src("(1+2)*3+4*(5+2)")
-    check_src("(1+2)*3+4*(5+2)**2")
+    check_src("1 << (10 | 1) << 1")
+    check_src("int | float | complex | None")
+    check_src("list[int | float | complex | None]")
+    check_src("list[int | float | complex | None, int | None]")
+    check_src("1 + 1")
+    check_src("1 + 2 / 3")
+    check_src("(1 + 2) / 3")
+    check_src("(1 + 2) * 3 + 4 * (5 + 2)")
+    check_src("(1 + 2) * 3 + 4 * (5 + 2) ** 2")
     check_src("~x")
     check_src("x and y")
     check_src("x and y and z")
@@ -1522,39 +1517,41 @@ def test_expressions_parens(subtests:Any) -> None:
     # cpython tests expected '(x**y)**z**q', 
     # but too much reasonning is needed to obtain this result,
     # because the power operator is reassociative...
-    check_src("(x**y)**(z**q)")
-    check_src("((x**y)**z)**q")
-    check_src("x>>y")
-    check_src("x<<y")
-    check_src("x>>y and x>>z")
-    check_src("x+y-z*q^t**k")
+    check_src("(x ** y) ** (z ** q)")
+    check_src("((x ** y) ** z) ** q")
+    check_src("x >> y")
+    check_src("x << y")
+    check_src("x >> y and x >> z")
+    check_src("x + y - z * q ^ t ** k")
     
-    check_src("flag&(other|foo)")
-    
-    # with astor (which adds a lot of parenthesis :/)
-    if sys.version_info>=(3,8):
-        check_src("(a := b)")
-    if sys.version_info>=(3,7):
-        check_src("(await x)")
-    check_src("(x if x else y)")
-    check_src("(lambda x: x)")
-    check_src("(lambda : int)()")
+    check_src("flag & (other | foo)")
+    check_src("(x if x else y).C")
     check_src("not (x == y)")
-    check_src("(x == (not y))")
-    check_src("(P * V if P and V else n * R * T)")
-    check_src("(lambda P, V, n: P * V == n * R * T)")
+
+    check_src("(a := b)")
     
+    if sys.version_info >= (3,11):
+        check_src("(lambda: int)()")
+    else:
+        check_src("(lambda : int)()")
+    
+    check_src("3 .__abs__()")
+    check_src("await x")
+    check_src("x if x else y")
+    check_src("lambda x: x")
+    check_src("x == (not y)")
+    check_src("P * V if P and V else n * R * T")
+    check_src("lambda P, V, n: P * V == n * R * T")
+
     check_src("f(**x)")
     check_src("{**x}")
 
-    check_src("(-1)**7")
-    check_src("(-1.0)**8")
-    check_src("(-1j)**6")
+    check_src("(-1) ** 7")
+    check_src("(-1.0) ** 8")
+    check_src("(-1j) ** 6")
     check_src("not True or False")
     check_src("True or not False")
 
-    check_src("(3).__abs__()")
-
     check_src("f(**([] or 5))")
     check_src("{**([] or 5)}")
     check_src("{**(~{})}")
@@ -1562,3 +1559,21 @@ def test_expressions_parens(subtests:Any) -> None:
     check_src("{**({} == {})}")
     check_src("{**{'y': 2}, 'x': 1, None: True}")
     check_src("{**{'y': 2}, **{'x': 1}}")
+
+
+def test_is_annotation_flag() -> None:
+    # the is_annotation attribute is added to all links when is_annotation=True is passed.
+    assert color(extract_expr(ast.parse('list[dict] + set()')), is_annotation=True) == '''<document source="pyval_repr">
+    <obj_reference is_annotation="True" refuri="list">
+        list
+    [
+    <wbr>
+    <obj_reference is_annotation="True" refuri="dict">
+        dict
+    ]
+     + 
+    <obj_reference is_annotation="True" refuri="set">
+        set
+    (
+    )
+'''
