@@ -4,6 +4,8 @@ from pathlib import Path
 import re
 import sys
 
+import pytest
+
 from pydoctor.options import Options
 from pydoctor import driver
 
@@ -67,6 +69,7 @@ def test_projectbasedir_absolute(tmp_path: Path) -> None:
     assert options.projectbasedirectory.is_absolute()
 
 
+@pytest.mark.skipif("platform.python_implementation() == 'PyPy' and platform.system() == 'Windows'")
 def test_projectbasedir_symlink(tmp_path: Path) -> None:
     """
     The --project-base-dir option, when given a path containing a symbolic link,
@@ -206,6 +209,7 @@ def test_main_return_non_zero_on_warnings() -> None:
     assert 'report_module.py:9: Cannot find link target for "BadLink"' in stream.getvalue()
 
 
+@pytest.mark.skipif("platform.python_implementation() == 'PyPy' and platform.system() == 'Windows'")
 def test_main_symlinked_paths(tmp_path: Path) -> None:
     """
     The project base directory and package/module directories are normalized
@@ -272,3 +276,58 @@ def test_make_intersphix(tmp_path: Path) -> None:
     assert [p.name for p in tmp_path.iterdir()] == ['objects.inv']
     assert inventory.is_file()
     assert b'Project: acme-lib\n# Version: 20.12.0-dev123\n' in inventory.read_bytes()
+
+def test_index_symlink(tmp_path: Path) -> None:
+    """
+    Test that the default behaviour is to create symlinks, at least on unix.
+
+    For windows users, this has not been a success, so we automatically fallback to copying the file now.
+    See https://github.com/twisted/pydoctor/issues/808, https://github.com/twisted/pydoctor/issues/720.
+    """
+    import platform
+    exit_code = driver.main(args=['--html-output', str(tmp_path), 'pydoctor/test/testpackages/basic/'])
+    assert exit_code == 0
+    link = (tmp_path / 'basic.html')
+    assert link.exists()
+    if platform.system() == 'Windows':
+        assert link.is_symlink() or link.is_file()
+    else:
+        assert link.is_symlink()
+
+def test_index_hardlink(tmp_path: Path) -> None:
+    """
+    Test for option --use-hardlink wich enforce the usage of harlinks.
+    """
+    exit_code = driver.main(args=['--use-hardlink', '--html-output', str(tmp_path), 'pydoctor/test/testpackages/basic/'])
+    assert exit_code == 0
+    assert (tmp_path / 'basic.html').exists()
+    assert not (tmp_path / 'basic.html').is_symlink()
+    assert (tmp_path / 'basic.html').is_file()
+
+
+def test_apidocs_help(tmp_path: Path) -> None:
+    """
+    Checks that the help page is well generated.
+    """
+    exit_code = driver.main(args=['--html-output', str(tmp_path), 'pydoctor/test/testpackages/basic/'])
+    assert exit_code == 0
+    help_page = (tmp_path / 'apidocs-help.html').read_text()
+    assert '>Search</h2>' in help_page
+
+def test_htmlbaseurl_option_all_pages(tmp_path: Path) -> None:
+    """
+    Check that the canonical link is included in all html pages, including summary pages.
+    """
+    exit_code = driver.main(args=[
+        '--html-base-url=https://example.com.abcde',
+        '--html-output', str(tmp_path), 'pydoctor/test/testpackages/basic/'])
+    assert exit_code == 0
+    for t in tmp_path.iterdir():
+        if not t.name.endswith('.html'):
+            continue
+        filename = t.name
+        if t.stem == 'basic':
+            filename = 'index.html' # since we have only one module it's linked as index.html
+        assert f'<link rel="canonical" href="https://example.com.abcde/{filename}"' in t.read_text(encoding='utf-8')
+    
+
