@@ -13,7 +13,7 @@ import pytest
 from twisted.web.template import Tag
 
 from pydoctor.options import Options
-from pydoctor import model, stanutils
+from pydoctor import model, stanutils, extensions
 from pydoctor.templatewriter import pages
 from pydoctor.utils import parse_privacy_tuple
 from pydoctor.sphinx import CacheT
@@ -37,8 +37,9 @@ class FakeDocumentable:
     A fake of pydoctor.model.Documentable that provides a system and
     sourceHref attribute.
     """
+    kind = None
     system: model.System
-    sourceHref = None
+    source_href = None
     filepath: str
 
 
@@ -62,7 +63,7 @@ def test_setSourceHrefOption(projectBaseDir: Path) -> None:
     mod.system = system
     system.setSourceHref(mod, projectBaseDir / "package" / "module.py")
 
-    assert mod.sourceHref == "http://example.org/trac/browser/trunk/package/module.py"
+    assert mod.source_href == "http://example.org/trac/browser/trunk/package/module.py"
 
 def test_htmlsourcetemplate_auto_detect() -> None:
     """
@@ -90,7 +91,7 @@ def test_htmlsourcetemplate_auto_detect() -> None:
         system = model.System(options)
 
         processPackage('basic', systemcls=lambda:system)
-        assert system.allobjects['basic.mod.C'].sourceHref == var_href
+        assert system.allobjects['basic.mod.C'].source_href == var_href
 
 def test_htmlsourcetemplate_custom() -> None:
     """
@@ -103,7 +104,7 @@ def test_htmlsourcetemplate_custom() -> None:
     system = model.System(options)
 
     processPackage('basic', systemcls=lambda:system)
-    assert system.allobjects['basic.mod.C'].sourceHref == "http://example.org/trac/browser/trunk/pydoctor/test/testpackages/basic/mod.py#n7"
+    assert system.allobjects['basic.mod.C'].source_href == "http://example.org/trac/browser/trunk/pydoctor/test/testpackages/basic/mod.py#n7"
 
 def test_initialization_default() -> None:
     """
@@ -340,7 +341,7 @@ def test_introspection_extension() -> None:
 
 testpackages = Path(__file__).parent / 'testpackages'
 
-@pytest.mark.skipif("platform.python_implementation() == 'PyPy'")
+@pytest.mark.skipif("platform.python_implementation() == 'PyPy' or platform.system() == 'Windows'")
 def test_c_module_text_signature(capsys:CapSys) -> None:
     
     c_module_invalid_text_signature = testpackages / 'c_module_invalid_text_signature'
@@ -379,7 +380,7 @@ def test_c_module_text_signature(capsys:CapSys) -> None:
         # cleanup
         subprocess.getoutput(f'rm -f {package_path}/*.so')
 
-@pytest.mark.skipif("platform.python_implementation() == 'PyPy'")
+@pytest.mark.skipif("platform.python_implementation() == 'PyPy' or platform.system() == 'Windows'")
 def test_c_module_python_module_name_clash(capsys:CapSys) -> None:
     c_module_python_module_name_clash = testpackages / 'c_module_python_module_name_clash'
     package_path = c_module_python_module_name_clash / 'mymod'
@@ -548,3 +549,28 @@ def test_name_defined() -> None:
     assert not innerFn.isNameDefined('F')
     assert not innerFn.isNameDefined('var')
     assert innerFn.isNameDefined('f')
+
+def test_priority_processor(capsys:CapSys) -> None:
+    system = model.System()
+    r = extensions.ExtRegistrar(system)
+    processor = system._post_processor
+    processor._post_processors.clear()
+
+    r.register_post_processor(lambda s:print('priority 200'), priority=200)
+    r.register_post_processor(lambda s:print('priority 100'))
+    r.register_post_processor(lambda s:print('priority 25'), priority=25)
+    r.register_post_processor(lambda s:print('priority 150'), priority=150)
+    r.register_post_processor(lambda s:print('priority 100 (bis)'))
+    r.register_post_processor(lambda s:print('priority 200 (bis)'), priority=200)
+
+    assert len(processor._post_processors)==6
+    processor.apply_processors()
+    assert len(processor.applied)==6
+    
+    assert capsys.readouterr().out.strip().splitlines() == ['priority 200', 
+                                                            'priority 200 (bis)',
+                                                            'priority 150',
+                                                            'priority 100',
+                                                            'priority 100 (bis)',
+                                                            'priority 25',
+                                                            ]
