@@ -184,3 +184,32 @@ mod:20: Cannot find link target for "notfound"
     assert re.match(_html_template_with_replacement.format(
         name='foum', package='Twisted', version='NEXT', replacement='mod.Baz.faam'
     ), class_html_text, re.DOTALL), class_html_text
+
+
+@twisted_deprecated_systemcls_param
+def test_twisted_python_deprecate_else_branch(capsys: CapSys, systemcls: Type[model.System]) -> None:
+    """
+    When @deprecated decorator is used within the else branch of a if block and the same name is defined
+    in the body branch, the name is not marked as deprecated.
+    """
+
+    mod = fromText('''
+    if sys.version_info>(3.8):
+        def foo():
+            ...
+        
+        class Bar:
+            ...
+    else:
+        from incremental import Version
+        @twisted.python.deprecate.deprecated(Version('python', 3, 8, 0), replacement='just use newer python version')
+        def foo():
+            ...
+        @twisted.python.deprecate.deprecated(Version('python', 3, 8, 0), replacement='just use newer python version')
+        class Bar:
+            ...
+    ''', systemcls=systemcls)
+
+    assert not capsys.readouterr().out
+    assert 'just use newer python version' not in test_templatewriter.getHTMLOf(mod.contents['foo'])
+    assert 'just use newer python version' not in test_templatewriter.getHTMLOf(mod.contents['Bar'])
\ No newline at end of file
