@@ -1,11 +1,15 @@
-class NewBaseClassA:
+class CommonBase(object):
+    def fullName(self):
+        ...
+
+class NewBaseClassA(CommonBase):
     def methodA(self):
         """
         This is method A.
         """
 
 
-class NewBaseClassB:
+class NewBaseClassB(CommonBase):
     def methodB(self):
         """
         This is method B.
@@ -19,14 +23,14 @@ class NewClassThatMultiplyInherits(NewBaseClassA, NewBaseClassB):
         """
 
 
-class OldBaseClassA:
+class OldBaseClassA(CommonBase):
     def methodA(self):
         """
         This is method A.
         """
 
 
-class OldBaseClassB:
+class OldBaseClassB(CommonBase):
     def methodB(self):
         """
         This is method B.
@@ -38,3 +42,6 @@ class OldClassThatMultiplyInherits(OldBaseClassA, OldBaseClassB):
         """
         This is method C.
         """
+
+class Diamond(OldClassThatMultiplyInherits, NewClassThatMultiplyInherits):
+    def newMethod(self):...
