@@ -7,14 +7,13 @@
 // Other required ressources like lunr.js, searchindex.json and all-documents.html are passed as URL
 //      to functions. This makes the code reusable outside of pydoctor build directory.    
 // Implementation note: Searches are designed to be launched synchronously, if lunrSearch() is called sucessively (while already running),
-// old promise will never resolves and the searhc worker will be restarted.
+// old promise will never resolves and the search worker will be restarted.
 
 // Hacky way to make the worker code inline with the rest of the source file handling the search.
 // Worker message params are the following: 
 // - query: string
 // - indexJSONData: dict
 // - defaultFields: list of strings
-// - autoWildcard: boolean
 let _lunrWorkerCode = `
 
 // The lunr.js code will be inserted here.
@@ -29,19 +28,17 @@ onmessage = (message) => {
     if (!message.data.defaultFields) {
         throw new Error('No default fields provided.');
     }
-    if (!message.data.hasOwnProperty('autoWildcard')){
-        throw new Error('No value for auto wildcard provided.');
-    }
+
     // Create index
     let index = lunr.Index.load(message.data.indexJSONData);
-    
+
     // Declare query function building 
     function _queryfn(_query){ // _query is the Query object
         // Edit the parsed query clauses that are applicable for all fields (default) in order
         // to remove the field 'kind' from the clause since this it's only useful when specifically requested.
         var parser = new lunr.QueryParser(message.data.query, _query)
         parser.parse()
-        var hasTraillingWildcard = false;
+
         _query.clauses.forEach(clause => {
             if (clause.fields == _query.allFields){
                 // we change the query fields when they are applicable to all fields
@@ -49,27 +46,44 @@ onmessage = (message) => {
                 // which should not be matched by default.
                 clause.fields = message.data.defaultFields;
             }
-            // clause.wildcard is actually always NONE due to https://github.com/olivernn/lunr.js/issues/495
-            // But this works...
-            if (clause.term.slice(-1) == '*'){
-                // we want to avoid the auto wildcard system only if a trailling wildcard is already added
-                // not if a leading wildcard exists
-                hasTraillingWildcard = true
-            }
+
         });
         // Auto wilcard feature, see issue https://github.com/twisted/pydoctor/issues/648
         var new_clauses = [];
-        if ((message.data.autoWildcard == true) && (hasTraillingWildcard == false)){
-            _query.clauses.forEach(clause => {
-                // Setting clause.wildcard is useless.
+
+        _query.clauses.forEach(clause => {
+            if (clause.presence === 1) { // ignore clauses that have explicit presence (+/-)
+                // Setting clause.wildcard is useless, and clause.wildcard is actually always NONE 
+                // due to https://github.com/olivernn/lunr.js/issues/495
                 // But this works...
-                let new_clause = {...clause}
-                new_clause.term = new_clause.term + '*'
-                clause.boost = 2
-                new_clause.boost = 0
-                new_clauses.push(new_clause)
-            });
-        }
+                if (clause.term.slice(-1) != '*'){
+                    let new_clause = {...clause}
+                    new_clause.term = new_clause.term + '*'
+                    clause.boost = 2
+                    new_clause.boost = 1
+                    new_clauses.push(new_clause)
+                }
+                
+                // Adding a leading wildcard if the dot is included as well.
+                // This should only apply to terms that are applicable to name-like fields. 
+                // so we refer to the default fields
+                if (clause.term.indexOf('.') != -1) {
+                    if (clause.term.slice(0,1) != '*'){
+                        let second_new_clause = {...clause}
+                        second_new_clause.boost = 1
+                        if (clause.term.slice(0,1) != '.'){ 
+                            second_new_clause.term = '.' + second_new_clause.term
+                        }
+                        second_new_clause.term = '*' + second_new_clause.term
+                        if (clause.term.slice(-1) != '*'){
+                            second_new_clause.term = second_new_clause.term + '*'
+                        }
+                        new_clauses.push(second_new_clause)
+                    }
+                }
+            }
+        });
+
         new_clauses.forEach(clause => {
             _query.clauses.push(clause)
         });
@@ -78,8 +92,8 @@ onmessage = (message) => {
     }
 
     // Launch the search
-    let results = index.query(_queryfn)
-    
+    var results = index.query(_queryfn)
+
     // Post message with results
     postMessage({'results':results});
 };
@@ -174,9 +188,8 @@ function _getWorkerPromise(lunJsSourceCode){ // -> Promise of a fresh worker to
  * @param lunrJsURL: URL pointing to a copy of lunr.js.
  * @param searchDelay: Number of miliseconds to wait before actually launching the query. This is useful to set for "search as you type" kind of search box
  *                     because it let a chance to users to continue typing without triggering useless searches (because previous search is aborted on launching a new one).
- * @param autoWildcard: Whether to automatically append wildcards to all query clauses when no wildcard is already specified. boolean.
- */
-function lunrSearch(query, indexURL, defaultFields, lunrJsURL, searchDelay, autoWildcard){
+*/
+function lunrSearch(query, indexURL, defaultFields, lunrJsURL, searchDelay){
     // Abort ongoing search
     abortSearch();
 
@@ -187,7 +200,7 @@ function lunrSearch(query, indexURL, defaultFields, lunrJsURL, searchDelay, auto
         searchEventsEnv.removeEventListener('abortSearch', this);
     });
 
-    // Pref:
+    // Perf:
     // Because this function can be called a lot of times in a very few moments, 
     // Actually launch search after a delay to let a chance to users to continue typing,
     // which would trigger a search abort event, which would avoid wasting a worker 
@@ -220,7 +233,6 @@ function lunrSearch(query, indexURL, defaultFields, lunrJsURL, searchDelay, auto
                 'query': query,
                 'indexJSONData': lunrIndexData,
                 'defaultFields': defaultFields,
-                'autoWildcard': autoWildcard, 
             }
             
             if (!_aborted){
